@@ -42,12 +42,18 @@ class CachedPropertyCollector(object):
     def __init__(self, client, managed_object_type, properties_list):
         super(CachedPropertyCollector, self).__init__()
         self._client = client
+        self._property_collector = None
         self._managed_object_type = managed_object_type
         self._properties_list = properties_list
         self._version = INITIAL_VERSION
         self._result = {}
         self._lock = Lock()
 
+    def __del__(self):
+        if self._property_collector is not None:
+            self._property_collector.Destroy()
+            self._property_collector = None
+
     def __repr__(self):
         args = (self.__class__.__name__, getattr(self, '_managed_object_type', None),
                 getattr(self, '_properties_list', []), getattr(self, '_version', repr('')))
@@ -72,7 +78,7 @@ class CachedPropertyCollector(object):
 
     @cached_method
     def _get_object_set(self):
-        return vim.ObjectSpec(obj=self._get_container_view(), selectSet=self._get_select_set())
+        return [vim.ObjectSpec(obj=self._get_container_view(), selectSet=self._get_select_set())]
 
     @cached_method
     def _get_prop_set(self):
@@ -80,14 +86,14 @@ class CachedPropertyCollector(object):
 
     @cached_method
     def _get_property_collector(self):
-        property_collector = self._client.service_content.propertyCollector.CreatePropertyCollector()
-        self._property_filter = property_collector.CreateFilter(self._get_property_filter_spec(), partialUpdates=True)
-        return property_collector
+        self._property_collector = self._client.service_content.propertyCollector.CreatePropertyCollector()
+        self._property_filter = self._property_collector.CreateFilter(self._get_property_filter_spec(), partialUpdates=True)
+        return self._property_collector
 
     @cached_method
     def _get_property_filter_spec(self):
         # http://vijava.sourceforge.net/vSphereAPIDoc/ver5/ReferenceGuide/vmodl.query.PropertyCollector.FilterSpec.html
-        return vim.PropertyFilterSpec(propSet=self._get_prop_set(), objectSet=[self._get_object_set()])
+        return vim.PropertyFilterSpec(propSet=self._get_prop_set(), objectSet=self._get_object_set())
 
     @cached_method
     def _get_select_set(self):
@@ -305,3 +311,22 @@ class VirtualMachinePropertyCollector(CachedPropertyCollector):
         select_set.append(self._create_traversal_spec(vim.ContainerView, 'container',
                           [select.name for select in select_set]))
         return select_set
+
+class TaskPropertyCollector(CachedPropertyCollector):
+    def __init__(self, client, tasks, properties=["info.state"]):
+        super(TaskPropertyCollector, self).__init__(client, vim.Task, properties)
+        self.tasks = tasks
+
+    def _get_object_set(self):
+        return [vim.ObjectSpec(obj=task) for task in self.tasks]
+
+    def iter_task_states_changes(self, timeout_in_seconds=None):
+        update = self._get_changes(time_in_seconds=timeout_in_seconds)
+        if update is None:
+            return
+        for filter_set in update.filterSet:
+            for obj_set in filter_set.objectSet:
+                task = obj_set.obj
+                for change in obj_set.changeSet:
+                    if change.name == 'info.state':    # we don't look for any other changes so this should be true
+                        yield task, change.val
